<?php

namespace App\Http\Controllers;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Models\MainCategory;
use Exception;


class CategoryController extends Controller
{

    public function getCategories()
    {
        try {
            $categories = MainCategory::with(['subCategories' => function($query) {
                $query->where('is_featured', true);
            }])
            ->whereNotIn('id', [2, 3])
            ->orderBy('id', 'ASC')
            ->get();

            $data = [];
            $featuredSubcategories = [];

            foreach ($categories as $category) {
                $categoryData = [
                    'main_category_id' => $category->id,
                    'category_name' => $category->category_name,
                    'subcategories' => []
                ];

                foreach ($category->subCategories as $subCategory) {
                    $subCategoryData = [
                        'sub_category_id' => $subCategory->id,
                        'sub_category' => $subCategory->sub_category,
                        'is_featured' => $subCategory->is_featured,
                        'sub_category_image' => $subCategory->sub_category_image
                    ];

                    $categoryData['subcategories'][] = $subCategoryData;

                    if ($subCategory->is_featured) {
                        $featuredSubcategories[] = $subCategoryData;
                    }
                }

                $data[] = $categoryData;
            }
            $status = $categories->isNotEmpty() || !empty($featuredSubcategories);


            return response()->json([
                'status' => $status,
                'categories' => $data,
                'featured_sub_categories' => $featuredSubcategories
            ], 200);

        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'An error occurred while fetching the categories.',
                'error' => $e->getMessage()
            ], 500);
        }
    }
    public function updateSubCategory(Request $request)
    {
        $response = [];

        try {
            $data = $request->all();

            $id = $data['sub_category_id'] ?? 0;
            $sub_category = $data['sub_category'] ?? "";
            $sub_category_image = $data['sub_category_image'] ?? "";
            $is_featured = $data['is_featured'] ?? 0;

            // Update the sub_category
            $affected = DB::table('sub_categories')
                ->where('id', $id)
                ->update([
                    'sub_category' => $sub_category,
                    'sub_category_image' => $sub_category_image,
                    'is_featured' => $is_featured
                ]);

            if ($affected) {
                $response['success'] = true;
                $response['message'] = "Sub-category updated successfully.";
            } else {
                $response['success'] = false;
                $response['message'] = "Failed to update sub-category.";
            }

            return response()->json($response, 200);
        } catch (Exception $e) {
            return response()->json([
                'success' => false,
                'message' => 'An error occurred while updating the sub-category.',
                'error' => $e->getMessage()
            ], 500);
        }
    }
}
