<?php

namespace App\Http\Controllers;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Exception;


class JsonProductController extends Controller
{
    public function uploadJsonProduct(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'json_file' => 'required|file|mimes:json',
        ]);

        if ($validator->fails()) {
            return response()->json(['error' => $validator->errors()], 400);
        }

        try {
            $file = $request->file('json_file');
            $fileContents = file_get_contents($file->getPathname());
            $jsonData = json_decode($fileContents, true);

            if (json_last_error() !== JSON_ERROR_NONE) {
                return response()->json(['error' => 'Error decoding JSON: ' . json_last_error_msg()], 400);
            }

            DB::beginTransaction();

            foreach ($jsonData as $item) {
                $price_net = $item['retail'] - $item['com_disc_amt'];

                DB::table('products')->updateOrInsert(
                    ['product_id' => $item['item_id']],
                    [
                        'product_name' => $item['item_name'],
                        'product_discount' => $item['com_disc_amt'],
                        'product_price' => $item['retail'],
                        'product_quantity' => ($item['active_ind'] == 'In-Active' || $item['qty'] < 1 || strtotime($item['modified_dte']) < strtotime('-6 months')) ? 0 : $item['qty'],
                        'product_net_price' => $price_net,
                        'active_ind' => $item['active_ind'],
                        'modified_dte' => $item['modified_dte']
                    ]
                );
            }

            DB::commit();
            return response()->json(['success' => 'Data imported successfully.'], 200);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(['error' => 'Error: ' . $e->getMessage()], 500);
        }
    }
}